import type { TSESTree } from '@typescript-eslint/types';
import { type ISuppression } from './bulk-suppressions-file';
interface IBulkSuppression {
    suppression: ISuppression;
    serializedSuppression: string;
}
interface IProblem {
    line: number;
    column: number;
    ruleId: string;
    suppressions?: {
        kind: string;
        justification: string;
    }[];
}
interface ILocation {
    line: number;
    column: number;
}
interface ISourceCode {
    ast: TSESTree.Node;
    getNodeByRangeIndex(index: number): TSESTree.Node;
    getIndexFromLoc(loc: ILocation): number;
    text: string;
    visitorKeys: Record<string, string[]>;
}
export interface ITraverser {
    traverse(node: TSESTree.Node, options: ITraverseOptions): void;
}
export interface ITraverseMethods {
    break(): void;
    skip(): void;
}
export interface ITraverseOptions {
    visitorKeys: Record<string, string[]>;
    enter(this: ITraverseOptions & ITraverseMethods, node: TSESTree.Node): void;
    leave(this: ITraverseOptions & ITraverseMethods, node: TSESTree.Node): void;
}
interface ILinterInternalSlots {
    lastSourceCode: ISourceCode | undefined;
    lastSuppressedMessages: IProblem[] | undefined;
}
export declare function getBulkSuppression(params: {
    serializedSuppressions: Set<string>;
    fileRelativePath: string;
    scopeId: string;
    problem: IProblem;
}): IBulkSuppression | undefined;
export declare function prune(): void;
export declare function write(): void;
/**
 * This returns a wrapped version of the "verify" function from ESLint's Linter class
 * that postprocesses rule violations that weren't suppressed by comments. This postprocessing
 * records suppressions that weren't otherwise suppressed by comments to be used
 * by the "suppress" and "prune" commands.
 */
export declare function extendVerifyFunction(originalFn: (this: unknown, ...args: unknown[]) => IProblem[] | undefined, getLinterInternalSlots: (linter: unknown) => ILinterInternalSlots, traverser: ITraverser): (this: unknown, ...args: unknown[]) => IProblem[] | undefined;
export {};
//# sourceMappingURL=bulk-suppressions-patch.d.ts.map