"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.extendVerifyFunction = exports.write = exports.prune = exports.getBulkSuppression = void 0;
const fs_1 = __importDefault(require("fs"));
const Guards = __importStar(require("./ast-guards"));
const constants_1 = require("./constants");
const bulk_suppressions_file_1 = require("./bulk-suppressions-file");
const ESLINTRC_FILENAMES = [
    '.eslintrc.js',
    '.eslintrc.cjs'
    // Several other filenames are allowed, but this patch requires that it be loaded via a JS config file,
    // so we only need to check for the JS-based filenames
];
const ESLINT_BULK_SUPPRESS_ENV_VAR_VALUE = process.env[constants_1.ESLINT_BULK_SUPPRESS_ENV_VAR_NAME];
const SUPPRESS_ALL_RULES = ESLINT_BULK_SUPPRESS_ENV_VAR_VALUE === '*';
const RULES_TO_SUPPRESS = ESLINT_BULK_SUPPRESS_ENV_VAR_VALUE
    ? new Set(ESLINT_BULK_SUPPRESS_ENV_VAR_VALUE.split(','))
    : undefined;
function getNodeName(node) {
    if (Guards.isClassDeclarationWithName(node)) {
        return node.id.name;
    }
    else if (Guards.isFunctionDeclarationWithName(node)) {
        return node.id.name;
    }
    else if (Guards.isClassExpressionWithName(node)) {
        return node.id.name;
    }
    else if (Guards.isFunctionExpressionWithName(node)) {
        return node.id.name;
    }
    else if (Guards.isNormalVariableDeclaratorWithAnonymousExpressionAssigned(node)) {
        return node.id.name;
    }
    else if (Guards.isNormalObjectPropertyWithAnonymousExpressionAssigned(node)) {
        return node.key.name;
    }
    else if (Guards.isNormalClassPropertyDefinitionWithAnonymousExpressionAssigned(node)) {
        return node.key.name;
    }
    else if (Guards.isNormalAssignmentPatternWithAnonymousExpressionAssigned(node)) {
        return node.left.name;
    }
    else if (Guards.isNormalMethodDefinition(node)) {
        return node.key.name;
    }
    else if (Guards.isTSEnumDeclaration(node)) {
        return node.id.name;
    }
    else if (Guards.isTSInterfaceDeclaration(node)) {
        return node.id.name;
    }
    else if (Guards.isTSTypeAliasDeclaration(node)) {
        return node.id.name;
    }
}
const eslintrcPathByFileOrFolderPath = new Map();
function findEslintrcFolderPathForNormalizedFileAbsolutePath(normalizedFilePath) {
    // Heft, for example, suppresses nested eslintrc files, so it can pass this environment variable to suppress
    // searching for the eslintrc file completely.
    let eslintrcFolderPath = process.env[constants_1.ESLINT_BULK_ESLINTRC_FOLDER_PATH_ENV_VAR_NAME];
    if (eslintrcFolderPath) {
        return eslintrcFolderPath;
    }
    const cachedFolderPathForFilePath = eslintrcPathByFileOrFolderPath.get(normalizedFilePath);
    if (cachedFolderPathForFilePath) {
        return cachedFolderPathForFilePath;
    }
    const normalizedFileFolderPath = normalizedFilePath.substring(0, normalizedFilePath.lastIndexOf('/'));
    const pathsToCache = [normalizedFilePath];
    findEslintrcFileLoop: for (let currentFolder = normalizedFileFolderPath; currentFolder; // 'something'.substring(0, -1) is ''
     currentFolder = currentFolder.substring(0, currentFolder.lastIndexOf('/'))) {
        const cachedEslintrcFolderPath = eslintrcPathByFileOrFolderPath.get(currentFolder);
        if (cachedEslintrcFolderPath) {
            return cachedEslintrcFolderPath;
        }
        pathsToCache.push(currentFolder);
        for (const eslintrcFilename of ESLINTRC_FILENAMES) {
            if (fs_1.default.existsSync(`${currentFolder}/${eslintrcFilename}`)) {
                eslintrcFolderPath = currentFolder;
                break findEslintrcFileLoop;
            }
        }
    }
    if (eslintrcFolderPath) {
        for (const checkedFolder of pathsToCache) {
            eslintrcPathByFileOrFolderPath.set(checkedFolder, eslintrcFolderPath);
        }
        return eslintrcFolderPath;
    }
    else {
        throw new Error(`Cannot locate an ESLint configuration file for ${normalizedFilePath}`);
    }
}
function getBulkSuppression(params) {
    const { fileRelativePath, serializedSuppressions, scopeId, problem } = params;
    const { ruleId: rule } = problem;
    const suppression = { file: fileRelativePath, scopeId, rule };
    const serializedSuppression = (0, bulk_suppressions_file_1.serializeSuppression)(suppression);
    const currentNodeIsSuppressed = serializedSuppressions.has(serializedSuppression);
    if (currentNodeIsSuppressed || SUPPRESS_ALL_RULES || (RULES_TO_SUPPRESS === null || RULES_TO_SUPPRESS === void 0 ? void 0 : RULES_TO_SUPPRESS.has(suppression.rule))) {
        // The suppressions object should already be empty, otherwise we shouldn't see this problem
        problem.suppressions = [
            {
                kind: 'bulk',
                justification: serializedSuppression
            }
        ];
        return {
            suppression,
            serializedSuppression
        };
    }
}
exports.getBulkSuppression = getBulkSuppression;
function prune() {
    for (const [eslintrcFolderPath, suppressionsConfig] of (0, bulk_suppressions_file_1.getAllBulkSuppressionsConfigsByEslintrcFolderPath)()) {
        if (suppressionsConfig) {
            const { newSerializedSuppressions, newJsonObject } = suppressionsConfig;
            const newSuppressionsConfig = {
                serializedSuppressions: newSerializedSuppressions,
                jsonObject: newJsonObject,
                newSerializedSuppressions: new Set(),
                newJsonObject: { suppressions: [] }
            };
            (0, bulk_suppressions_file_1.writeSuppressionsJsonToFile)(eslintrcFolderPath, newSuppressionsConfig);
        }
    }
}
exports.prune = prune;
function write() {
    for (const [eslintrcFolderPath, suppressionsConfig] of (0, bulk_suppressions_file_1.getAllBulkSuppressionsConfigsByEslintrcFolderPath)()) {
        if (suppressionsConfig) {
            (0, bulk_suppressions_file_1.writeSuppressionsJsonToFile)(eslintrcFolderPath, suppressionsConfig);
        }
    }
}
exports.write = write;
function binarySearch(arr, target, low, high) {
    while (low <= high) {
        // eslint-disable-next-line no-bitwise
        const mid = (low + high) >> 1;
        const midVal = arr[mid];
        if (midVal === target) {
            return mid;
        }
        else if (midVal < target) {
            low = mid + 1;
        }
        else {
            high = mid - 1;
        }
    }
    // eslint-disable-next-line no-bitwise
    return ~low;
}
function getScopeIdMap(traverser, sourceCode, positions) {
    const scopeIdMap = new Map();
    if (positions.length === 0) {
        return scopeIdMap;
    }
    let low = 0;
    let high = positions.length - 1;
    const max = positions[high];
    const highStack = [];
    traverser.traverse(sourceCode.ast, {
        visitorKeys: sourceCode.visitorKeys,
        enter(node) {
            const startInclusive = node.range[0];
            if (startInclusive > max) {
                // Future nodes irrelevant. We can break out of the traversal.
                return this.break();
            }
            highStack.push(high);
            const endInclusive = node.range[1] - 1;
            let newLow = binarySearch(positions, startInclusive, low, high);
            let newHigh = binarySearch(positions, endInclusive, low, high);
            if (newLow < 0) {
                // Start point before the current minimum gets excluded
                newLow = ~newLow;
            }
            if (newHigh < 0) {
                // End point above the current maximum gets excluded
                newHigh = ~newHigh - 1;
            }
            if (newLow > newHigh) {
                return this.skip();
            }
            low = newLow;
            high = newHigh;
            const currentNodeName = getNodeName(node);
            if (currentNodeName) {
                const existingScopeId = scopeIdMap.get(positions[low]);
                const newScopeId = `${existingScopeId !== null && existingScopeId !== void 0 ? existingScopeId : ''}.${currentNodeName}`;
                for (let i = low; i <= high; i++) {
                    scopeIdMap.set(positions[i], newScopeId);
                }
            }
        },
        leave(node) {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            const oldHigh = highStack.pop();
            // Traversal has monotonically increasing positions, so no need to reduce low
            high = oldHigh;
        }
    });
    return scopeIdMap;
}
/**
 * This returns a wrapped version of the "verify" function from ESLint's Linter class
 * that postprocesses rule violations that weren't suppressed by comments. This postprocessing
 * records suppressions that weren't otherwise suppressed by comments to be used
 * by the "suppress" and "prune" commands.
 */
function extendVerifyFunction(originalFn, getLinterInternalSlots, traverser) {
    return function (...args) {
        var _a;
        const problems = originalFn.apply(this, args);
        if (!problems) {
            return problems;
        }
        const internalSlots = getLinterInternalSlots(this);
        const { lastSourceCode } = internalSlots;
        if (!lastSourceCode) {
            // We don't have a file for context, nothing we can do here.
            return problems;
        }
        if (args.length < 3) {
            throw new Error('Expected at least 3 arguments to Linter.prototype.verify');
        }
        const fileNameOrOptions = args[2];
        const filename = typeof fileNameOrOptions === 'string' ? fileNameOrOptions : fileNameOrOptions.filename;
        let { lastSuppressedMessages } = internalSlots;
        const positions = [];
        const problemToPositionMap = new Map();
        for (const problem of problems) {
            const { line, column } = problem;
            if (typeof line === 'number' && typeof column === 'number') {
                const position = lastSourceCode.getIndexFromLoc({ line, column: column - 1 });
                problemToPositionMap.set(problem, position);
                positions.push(position);
            }
        }
        positions.sort((x, y) => x - y);
        const scopeIdMap = getScopeIdMap(traverser, lastSourceCode, positions);
        const normalizedFileAbsolutePath = filename.replace(/\\/g, '/');
        const eslintrcDirectory = findEslintrcFolderPathForNormalizedFileAbsolutePath(normalizedFileAbsolutePath);
        const fileRelativePath = normalizedFileAbsolutePath.substring(eslintrcDirectory.length + 1);
        const config = (0, bulk_suppressions_file_1.getSuppressionsConfigForEslintrcFolderPath)(eslintrcDirectory);
        const { newSerializedSuppressions, serializedSuppressions, jsonObject: { suppressions }, newJsonObject: { suppressions: newSuppressions } } = config;
        const filteredProblems = [];
        for (const problem of problems) {
            const position = problemToPositionMap.get(problem);
            const scopeId = position === undefined ? '.' : (_a = scopeIdMap.get(position)) !== null && _a !== void 0 ? _a : '.';
            const bulkSuppression = getBulkSuppression({
                fileRelativePath,
                serializedSuppressions,
                scopeId,
                problem
            });
            if (!bulkSuppression) {
                filteredProblems.push(problem);
                continue;
            }
            const { serializedSuppression, suppression } = bulkSuppression;
            if (!newSerializedSuppressions.has(serializedSuppression)) {
                newSerializedSuppressions.add(serializedSuppression);
                newSuppressions.push(suppression);
                suppressions.push(suppression);
                if (!lastSuppressedMessages) {
                    lastSuppressedMessages = [];
                    internalSlots.lastSuppressedMessages = lastSuppressedMessages;
                }
                lastSuppressedMessages.push(problem);
            }
        }
        return filteredProblems;
    };
}
exports.extendVerifyFunction = extendVerifyFunction;
//# sourceMappingURL=bulk-suppressions-patch.js.map