"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
Object.defineProperty(exports, "__esModule", { value: true });
const _patch_base_1 = require("../_patch-base");
const path_utils_1 = require("./path-utils");
const bulk_suppressions_patch_1 = require("./bulk-suppressions-patch");
const constants_1 = require("./constants");
function apply() {
    if (!_patch_base_1.eslintFolder) {
        console.error('@rushstack/eslint-patch/eslint-bulk-suppressions: Could not find ESLint installation to patch.');
        process.exit(1);
    }
    const eslintBulkDetectEnvVarValue = process.env[constants_1.ESLINT_BULK_DETECT_ENV_VAR_NAME];
    if (eslintBulkDetectEnvVarValue === 'true' || eslintBulkDetectEnvVarValue === '1') {
        (0, path_utils_1.findAndConsoleLogPatchPathCli)();
        process.exit(0);
    }
    if (process.env[constants_1.ESLINT_BULK_ENABLE_ENV_VAR_NAME] === 'false') {
        return;
    }
    const { linterPath, traverserPath } = (0, path_utils_1.getPathsToPatch)();
    const { Linter, getLinterInternalSlots } = require(linterPath);
    const Traverser = require(traverserPath);
    const { verify: originalVerify } = Linter.prototype;
    Linter.prototype.verify = (0, bulk_suppressions_patch_1.extendVerifyFunction)(originalVerify, getLinterInternalSlots, Traverser);
}
apply();
//# sourceMappingURL=index.js.map